<?php
declare(strict_types=1);

namespace IntiTest\Provider\Service;

use Inti\Provider\ProviderInterface;
use Inti\Provider\Service\ProviderManager;
use IntiTest\Provider\Mock;
use Laminas\ServiceManager\ServiceManager;
use PHPUnit\Framework\TestCase;

/**
 * Class ProviderManagerTest
 *
 * @package IntiTest\Provider\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
final class ProviderManagerTest extends TestCase
{
    private ProviderManager $manager;

    protected function setUp(): void
    {
        $this->manager = new ProviderManager(new ServiceManager(), [
            'factories' => [
                'custom_type' => Mock\CustomProviderFactory::class,
                'other_type'  => Mock\CustomProviderFactory::class,
            ],
            'invokables' => [
                Mock\MockProvider::class,
            ],
        ]);
    }

    public function testCanGetAllProviders()
    {
        $providers = $this->manager->getProviders();
        $this->assertCount(3, $providers);
    }

    public function testCanGetProvidersOfType()
    {
        $this->assertCount(2, $this->manager->getProviders(Mock\CustomProviderInterface::class));
    }

    public function testCanGetProvidersForType()
    {
        $this->assertCount(2, $this->manager->getProviders(
            Mock\CustomProviderInterface::class,
            Mock\CustomTypeInterface::class
        ));
    }

    public function testCanGetProviderForType()
    {
        $this->assertTrue($this->manager->canProvide(Mock\CustomProviderInterface::class, 'custom_type'));
        $this->assertTrue($this->manager->canProvide(ProviderInterface::class, 'mock_type'));
        $this->assertNotNull($this->manager->getProvider(Mock\CustomProviderInterface::class, 'other_type'));
        $this->assertNull($this->manager->getProvider(Mock\CustomProviderInterface::class, 'not_provided'));
    }
}
